#include "qwayland-server-qtkey-extension.h"

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    qt_key_extension::qt_key_extension(struct ::wl_client *client, int id)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id);
    }

    qt_key_extension::qt_key_extension(struct ::wl_display *display)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display);
    }

    qt_key_extension::qt_key_extension()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_key_extension::~qt_key_extension()
    {
    }

    void qt_key_extension::init(struct ::wl_client *client, int id)
    {
        m_resource = bind(client, id);
    }

    qt_key_extension::Resource *qt_key_extension::add(struct ::wl_client *client, int id)
    {
        Resource *resource = bind(client, id);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_key_extension::init(struct ::wl_display *display)
    {
        m_global = wl_display_add_global(display, &::qt_key_extension_interface, this, bind_func);
    }

    qt_key_extension::Resource *qt_key_extension::key_extension_allocate()
    {
        return new Resource;
    }

    void qt_key_extension::key_extension_bind_resource(Resource *)
    {
    }

    void qt_key_extension::key_extension_destroy_resource(Resource *)
    {
    }

    void qt_key_extension::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        Q_UNUSED(version);
        static_cast<qt_key_extension *>(data)->add(client, id);
    }

    void qt_key_extension::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_key_extension *that = resource->key_extension;
        that->m_resource_map.remove(resource->client(), resource);
        that->key_extension_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_key_extension::Resource *qt_key_extension::bind(struct ::wl_client *client, uint32_t id)
    {
        Resource *resource = key_extension_allocate();
        resource->key_extension = this;

        struct ::wl_resource *handle = wl_client_add_object(client, &::qt_key_extension_interface, &m_qt_key_extension_interface, id, resource);

        handle->destroy = destroy_func;
        resource->handle = handle;
        key_extension_bind_resource(resource);
        return resource;
    }

    const struct ::qt_key_extension_interface qt_key_extension::m_qt_key_extension_interface = {
        qt_key_extension::handle_dummy
    };

    void qt_key_extension::key_extension_dummy(Resource *)
    {
    }


    void qt_key_extension::handle_dummy(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_key_extension *>(r->key_extension)->key_extension_dummy(
            r);
    }

    void qt_key_extension::send_qtkey(uint32_t time, uint32_t type, uint32_t key, uint32_t modifiers, uint32_t nativeScanCode, uint32_t nativeVirtualKey, uint32_t nativeModifiers, const QString &text, uint32_t autorepeat, uint32_t count)
    {
        send_qtkey(
            m_resource->handle,
            time,
            type,
            key,
            modifiers,
            nativeScanCode,
            nativeVirtualKey,
            nativeModifiers,
            text,
            autorepeat,
            count);
    }

    void qt_key_extension::send_qtkey(struct ::wl_resource *resource, uint32_t time, uint32_t type, uint32_t key, uint32_t modifiers, uint32_t nativeScanCode, uint32_t nativeVirtualKey, uint32_t nativeModifiers, const QString &text, uint32_t autorepeat, uint32_t count)
    {
        qt_key_extension_send_qtkey(
            resource,
            time,
            type,
            key,
            modifiers,
            nativeScanCode,
            nativeVirtualKey,
            nativeModifiers,
            text.toUtf8().constData(),
            autorepeat,
            count);
    }

}

QT_END_NAMESPACE
